#!/usr/bin/env python
# encoding: utf-8

"""
funcutils provided some useful functions.
"""

import inspect
import sys
import thread


def is_magic_method(method):
    """
    Check a method if is magic method

    :param method: method of a class
    """
    if not inspect.ismethod(method):
        return False
    func_name = method.im_func.func_name
    if func_name.startswith("__") and func_name.endswith("__"):
        return True
    return False


def set_default_attr(obj, name, value):
    """
    Set attribute to obj if attribute is not exists

    :param obj: object
    :param name: attribure name
    :param value: attribure value
    :return: if is attribute not exists
    """
    if not hasattr(obj, name):
        setattr(obj, name, value)
        return value
    return getattr(obj, name, value)


def iter_attrs(
    obj, include_fields=(), exclude_fields=(),
    public_only=True, exclude_methods=True,
):
    """
    iter attributes of obj

    :param obj: object
    :param include_fields: include all this fields if exists
    :param exclude_fields: exclude this fields
    :param public_only: choice public attr only
    :param exclude_methods: exclude methods
    """
    obj_type = type(obj)
    obj_type_name = obj_type.__name__
    for attr, val in inspect.getmembers(obj):
        if attr not in include_fields:
            if (
                attr in exclude_fields or
                attr.startswith("__") or
                attr.startswith("_%s__" % obj_type_name) or
                (public_only and attr.startswith("_")) or
                (exclude_methods and inspect.ismethod(val))
            ):
                continue
        yield attr, val


def parent_frame():
    """
    Return parent frame of invoker.
    """
    frames = sys._current_frames()
    current_frame = frames[thread.get_ident()]
    invoker_frame = current_frame.f_back
    return invoker_frame and invoker_frame.f_back


def export_module(name, env_locals=None):
    """
    Export module's attributes to invoker's locals, like:
        from xxx import *

    :param name: module name
    :param env_locals: invoker locals
    """
    import importlib

    if env_locals is None:
        frame = parent_frame()
        env_locals = frame.f_locals

    module = importlib.import_module(name)
    all_attrs = list(getattr(
        module, "__all__",
        (i for i in dir(module) if not i.startswith("_"))
    ))

    for i in all_attrs:
        env_locals[i] = getattr(module, i)

    return len(all_attrs)



#!/usr/bin/env python
# encoding: utf-8

from itertools import groupby
from operator import itemgetter


def get_real_bases(bases):
    """
    Get real bases for types argument.
    >>> get_real_bases(None) #=> (object,)
    >>> get_real_bases(TypeA) #=> (TypeA,)
    >>> get_real_bases([TypeA, TypeB]) #=> (TypeA, TypeB)

    :param bases: type or type sequence
    """
    if bases is None:
        return (object,)
    if isinstance(bases, type):
        bases = (bases,)
    return tuple(bases)


def subtype(name, bases=None, attrs=None):
    """
    A easier way to create a type inherited from bases(default:object)
    with specified attrs.

    :param name: name of new type
    :param bases: bases class of new type
    :param attrs: class attributes of new type
    """
    return type(name, get_real_bases(bases), dict(attrs or {}))


class TypeFactory(object):
    """
    Create your type from this factory.
    >>> types_factory.NewType()
    equals:
    >>> subtype("NewType")
    """
    def __getattr__(self, name):
        return lambda bases=None, attrs=None: subtype(
            name, bases, attrs,
        )

types_factory = TypeFactory()


def subexception(name, bases=None, attrs=None):
    """
    A easier way to create an Exception

    :param name: name of new exception
    :param bases: bases class of new exception
    :param attrs: class attributes of new exception
    """
    return subtype(name, bases or [Exception], attrs)


class ExceptionFactory(object):
    """
    Create your type by this factory.
    >>> exceptions_factory.NewError()
    equals:
    >>> subexception("NewError")
    """
    def __getattr__(self, name):
        return lambda bases=None, attrs=None: subexception(
            name, bases, attrs,
        )

exceptions_factory = ExceptionFactory()


class SimpleExceptions(object):
    """
    Create and cached a simple exception.
    """
    def __getattr__(self, name):
        exception = subexception(name)
        setattr(self, name, exception)
        return exception


def freezed_attrs(attrs):
    """
    Decorator the declare attributes of cls is freezed.
    Attributes in attrs can only assigned once as
    initialization(usually is in __init__).

    :param attrs: attribute list
    """
    def setattr_hook(cls):
        def __setattr__(self, name, val):
            if name not in attrs or hasattr(self, name):
                raise AttributeError("attribute %s is not writable" % name)
            return super(cls, self).__setattr__(name, val)

        return subtype(
            cls.__name__, cls,
            {
                "__doc__": cls.__doc__,
                "__setattr__": __setattr__,
            }
        )
    return setattr_hook


class Constants(object):
    """
    The base class of constants
    """
    def __new__(cls, mappings, attrs):
        consts_index = {
            val: tuple(i[0] for i in item)
            for val, item in groupby(mappings.items(), itemgetter(1))
        }

        __name__ = attrs.pop("name", cls.__name__)

        def __iter__(self):
            return mappings.iteritems()
        hook = attrs.pop("iter_hook", None)
        if hook:
            __iter__ = hook(__iter__)

        def __getitem__(self, val):
            return consts_index.get(val)
        hook = attrs.pop("getitem_hook", None)
        if hook:
            __getitem__ = hook(__getitem__)

        def __len__(self):
            return len(mappings)
        hook = attrs.pop("len_hook", None)
        if hook:
            __len__ = hook(__len__)

        def __setattr__(self, name, val):
            if name not in mappings or hasattr(self, name):
                raise AttributeError("attribute %s is not writable" % name)
            return super(cls, self).__setattr__(name, val)
        hook = attrs.pop("setattr_hook", None)
        if hook:
            __setattr__ = hook(__setattr__)

        constants = super(Constants, cls).__new__(subtype(
            __name__, cls,
            {
                "__doc__": cls.__doc__,
                "__iter__": __iter__,
                "__getitem__": __getitem__,
                "__len__": __len__,
                "__setattr__": __setattr__,
            }
        ))

        return constants

    def __init__(self, mappings, attrs):
        for k, v in mappings.items():
            setattr(self, k, v)


def constants(**kwg):
    """
    Declare some constants.
    """
    return Constants(kwg, {"name": "ConstantSet"})


def _enums_getitem_hook(method):
    def getitem(self, name):
        return method(self, name)[0]
    return getitem


def enums(*values):
    """
    Declare some enumerations.
    """
    return Constants(
        {k: i for i, k in enumerate(values)},
        {"name": "EnumerationSet", "getitem_hook": _enums_getitem_hook}
    )



#!/usr/bin/env python
# encoding: utf-8

import time


class Stopwatch(object):
    """
    A sadness timer
    """
    def __init__(self):
        self.start_on = None
        self._duration = 0
        self._enable = False

    @property
    def enable(self):
        return self._enable

    @enable.setter
    def enable(self, val):
        if val:
            self.start_on = time.time()
            self._enable = True
        else:
            self._duration = self.duration
            self._enable = False

    @property
    def duration(self):
        if self.enable:
            return time.time() - self.start_on + self._duration
        return self._duration

    def __enter__(self):
        self.enable = True
        return self

    def __exit__(self, typ, val, trbk):
        self.enable = False



#!/usr/bin/env python
# encoding: utf-8

"""
decoratorutils provided some useful decorators.
"""

import functools
import logging
import types

logger = logging.getLogger(__file__)


class DescriptorBase(object):

    def __init__(self, func):
        self.func = func


def call_trace(loger=None, name=None):
    """
    Trace a fun call with loger.
    :param loger: logging.Logger
    :param name: function name
    """
    def _(func):
        fname = name or func.func_name

        @functools.wraps(func)
        def f(*args, **kwg):
            fid = id(args) ^ id(kwg)
            loger.info("%s[%d] call", fname, fid)
            try:
                res = func(*args, **kwg)
                loger.info("%s[%s] return", fname, fid)
                return res
            except Exception as err:
                loger.info("%s[%d] got error: %s", fname, fid, err)
                raise
        return f
    return _


def with_lock(lock):
    """
    Call function with a lock
    :param lock: lock context manager
    """
    def _(func):
        @functools.wraps(func)
        def f(*args, **kwg):
            with lock:
                return func(*args, **kwg)
        return f
    return _


class CachedProperty(DescriptorBase):
    """
    Call func at first time and cached the result, return result
    immediately later.
    """

    def __get__(self, instance, cls):
        if not hasattr(self, "cached_result"):
            result = types.MethodType(self.func, instance, cls)()
            setattr(instance, self.func.func_name, result)
        return result
cachedproperty = CachedProperty


def chainingmethod(func):
    """
    Declare a method is a chaining method which return self as returned value
    :param func: real func
    """
    @functools.wraps(func)
    def chaining(self, *args, **kwg):
        func(self, *args, **kwg)
        return self
    return chaining


def retry(num, errors=Exception):
    """
    Recall func when catched target errors at most num times
    :param num: maximum retry number, 0 for forever.
    :param errors: catch target errors
    """
    if num < 0:
        raise ValueError("num:%d can not less than 0" % num)

    def foo(func):
        @functools.wraps(func)
        def baz(*args, **kwg):
            i = num and num + 1
            while True:
                try:
                    return func(*args, **kwg)
                except errors:
                    i -= 1
                    if i == 0:
                        raise
        return baz
    return foo


def withmanager(ctxmgr, *ctxargs, **ctxkwg):
    """
    A decorator call function with ctxmgr, if ctxmgr is callable
    reset ctxmgr as ctxmgr(*ctxargs, **ctxkwg) every time.
    :param ctxmgr: context manager
    :param ctxargs: position argument
    :param ctxkwg: key word argument
    """
    def decorator(func):
        @functools.wraps(func)
        def helper(*args, **kwg):
            mgr = ctxmgr
            if callable(ctxmgr):
                mgr = ctxmgr(*ctxargs, **ctxkwg)
            with mgr:
                return func(*args, **kwg)

        return helper
    return decorator


def onerror_return(default_val, errors=Exception, callback=logger.warning):
    """
    return the default value when catch errors
    :param default_val: default value
    :param errors: catch exceptions(default: Exception)
    :param callback: callback when catched a exception(default: logger.warning)
    :return: func result if success else default_val
    """
    def foo(func):
        @functools.wraps(func)
        def baz(*args, **kwg):
            try:
                return func(*args, **kwg)
            except errors as err:
                if callback:
                    callback(err)
            return default_val
        return baz
    return foo


def call_immediately(checker=True):
    """
    Call func immediately and return the result instead func name.

    :param args: position arguments
    :param kwg: key word arguments
    """
    if isinstance(checker, types.BooleanType):
        checker = (lambda: checker)

    def invoker(func):
        if checker():
            return func()
        return func
    return invoker


def method_decorator(decorator):
    """
    Make a simple decorator to adapt class method

    :param decorator: simple decorator
    """
    def foo(func):
        def bar(_, *args, **kwg):
            method = func.__get__(_, type(_))
            return decorator(method)(*args, **kwg)
        return bar
    return foo


def withattr(**kwg):
    """
    Set attributes to a func, example:
    >>> Class Model(DjangoModel):
    ...     @withattr(alters_data=True)
    ...     def delete(self):
    ...         delete(self)

    :param kwg: attributes dict
    """
    def foo(func):
        for name, value in kwg.items():
            setattr(func, name, value)
        return func

    return foo


class AllowUnboundMethod(DescriptorBase):
    """
    Allow a method called by class
    """

    def __get__(self, instance, cls):
        if instance is not None:
            return types.MethodType(self.func, instance, cls)
        return types.UnboundMethodType(self.func, cls, cls)
allow_unbound_method = AllowUnboundMethod


class ClassProperty(DescriptorBase):
    """
    A class property
    """

    def __get__(self, instance, cls):
        return types.UnboundMethodType(self.func, cls, cls)()
classproperty = ClassProperty



#!/usr/bin/env python
# encoding: utf-8

from ycyc.base.contextutils import *
from ycyc.base.allowfail import AllowFail
from ycyc.base.filetools import (
    cd, safe_open_for_read, safe_open_for_update, safe_open_for_write,
)
from ycyc.base.logutils import log_disable



#!/usr/bin/env python
# coding=utf-8


class AsyncResultProxy(object):
    def __init__(self, async_result):
        self._async_result = async_result

    def __getattr__(self, name):
        result = self._async_result.get()
        return getattr(self._async_result, name)



#!/usr/bin/env python
# encoding: utf-8

import threading


class BlockSubscriber(object):
    """
    Block until event happend
    """

    def __init__(self, checker=None):
        self.block_cond = threading.Condition()
        self.checker = checker or (lambda *a, **k: True)

    def __call__(self, *args, **kwg):
        if not self.checker(*args, **kwg):
            return
        with self.block_cond:
            self.block_cond.notify()

    def wait(self, timeout=None):
        """
        Wait until event happend
        """
        with self.block_cond:
            self.block_cond.wait(timeout)



#!/usr/bin/env python
# encoding: utf-8

from collections import MutableMapping
import cPickle

from .base import RedisKit


class RedisHashDict(MutableMapping, RedisKit):
    Convertor = cPickle

    def __init__(self, connection, name, *args, **kwargs):
        self.connection = connection
        self.name = name
        super(RedisHashDict, self).__init__(*args, **kwargs)

    def __setitem__(self, key, value):
        self.connection.hset(self.name, key, self.Convertor.dumps(value))

    def __getitem__(self, key):
        value = self.connection.hget(self.name, key)
        if value is None:
            raise KeyError("%s not found" % key)
        return self.Convertor.loads(value)

    def __delitem__(self, key):
        if self.connection.hdel(self.name, key) == 0:
            raise KeyError("%s not found" % key)

    def __iter__(self):
        return iter(self.connection.hkeys(self.name))

    def __len__(self):
        return self.connection.hlen(self.name)

    def __contains__(self, key):
        return self.connection.hexists(self.name, key)



#!/usr/bin/env python
# encoding: utf-8

import ast
from itertools import chain
from functools import partial

from ycyc.base import contextutils
from ycyc.base.iterutils import dict_merge, getitems

NameParseError = type("NameParseError", (NameError,), {})


class SafeCalc(ast.NodeTransformer):
    """
    Python expression safe calculator
    """
    globals = {
        "locals": None, "globals": None, "__name__": None,
        "__file__": None, "__builtins__": None, "True": True,
        "False": False,
    }

    def __init__(self, locals, allow_attr=True, timeout=0, interval=None):
        self.locals = locals
        if timeout:
            self.timeout = partial(contextutils.timeout, timeout, interval)
        else:
            self.timeout = contextutils.nothing
        self.allow_attr = allow_attr

    def visit_Call(self, node):
        if isinstance(node.func, ast.Name):
            name = node.func.id
            if name not in self.locals and name not in self.globals:
                raise NameParseError("%s not found" % name)
        self.generic_visit(node)
        return node

    def visit_Attribute(self, node):
        name = node.attr
        if not self.allow_attr or name.startswith("__"):
            raise NameParseError("%s not allow" % name)
        self.generic_visit(node)
        return node

    def __call__(self, expr):
        ast_node = ast.parse(expr, mode="eval")
        ast_node = self.visit(ast_node)
        ast_node = ast.fix_missing_locations(ast_node)
        code = compile(ast_node, "<string>", mode="eval")
        with self.timeout():
            return eval(code, self.globals, self.locals)


def safecalc(expr, locals=None, **vars):
    """
    A quick function to calculate Python expression

    :param expr: Python expression
    :param locals: local env
    :param vars: variables
    :return: value
    """
    calc = SafeCalc(dict_merge((vars, locals or {})))
    return calc(expr)


def select(dct_object, *args, **kwg):
    """
    Select some items from complex dict object.
    """
    return {
        k: getitems(dct_object, i)
        for k, i in chain(enumerate(args), kwg.iteritems())
    }



#!/usr/bin/env python
# encoding: utf-8

"""
iterutils provided some utils to operate a iterable or indexable object.
"""

import itertools
import collections
import logging

logger = logging


def getitems(obj, items, default=None):
    """
    Get items from obj
    :param obj: object
    :param items: string or iterable object
    :param default: default value
    :return: obj item value if existed, otherwise default value
    """
    if isinstance(items, (str, unicode)):
        items = (items,)

    try:
        return reduce(lambda x, i: x[i], items, obj)
    except (IndexError, KeyError):
        return default


def getattrs(obj, attrs, default=None):
    """
    Get attrs from obj
    :param obj: object
    :param attrs: string or iterable object
    :param default: default value
    :return: obj attr value if existed, otherwise default value
    """
    if not attrs:
        return obj

    if isinstance(attrs, (str, unicode)):
        attrs = (attrs,)

    try:
        return reduce(lambda x, n: getattr(x, n), attrs, obj)
    except AttributeError:
        return default


def iterable(obj):
    """
    Check obj if is iterable
    :param obj: object
    :return: boolean
    """
    return hasattr(obj, "__iter__")


def getnext(iterator, default=None):
    """
    Get next item of iterable object
    :param iterator: object
    :param default: default value
    :return: next item of iterator till the end, otherwise default value
    """
    try:
        return next(iterator)
    except StopIteration:
        return default


def getfirst(obj, default=None):
    """
    Get first item of iterable object
    :param obj: object
    :param default: default value
    :return: first item of obj if obj is not empty, otherwise default value
    """
    if not obj:
        return default
    return getnext(iter(obj), default)


def groupby(iterable, keyfunc):
    """
    Group iterable by keyfunc
    :param iterable: iterable obj
    :param keyfunc: group by keyfunc
    :return: dict {<keyfunc result>: [item1, item2]}
    """
    data = sorted(iterable, key=keyfunc)
    return {
        k: list(g)
        for k, g in itertools.groupby(data, keyfunc)
    }


def mkparts(sequence, indices=None):
    """
    Make some parts from sequence by indices
    :param sequence: indexable object
    :param indices: index list
    :return: [seq_part1, seq_part2, ...]
    """
    indices = indices or [1]
    result_list = collections.deque()
    start = 0
    seq_len = len(sequence)
    for end in indices:
        if end < 0:
            end = seq_len + end
        if end < start:
            raise ValueError("end index is less than start index")
        result_list.append(sequence[start:end])
        start = end
    result_list.append(sequence[start:])
    return tuple(result_list)


def get_single_item(obj, default=None, logger=logger):
    """
    Get the single item from obj, if obj has more than 1 items, it
    will log a warning record.
    :param obj: iterable obj
    :param default: default value if obj is empty
    :return: obj first item
    """
    if obj and len(obj) > 1:
        logger.warning(
            "iterable object has more than 1 items, %s actually.", len(obj)
        )
    return getfirst(obj, default)


def dict_merge(dicts):
    """
    Merge dict list to a dict object
    :param dicts: dict list
    :return: dict
    """
    result = {}

    for d in dicts:
        for k, v in d.items():
            if k not in result:
                result[k] = v

    return result


def flatten(sequence_list, cls=list):
    """
    Flatten one level of nesting
    :param sequence_list: list of sequence
    :param cls: create instance of cls by flatten_gen
    :return: cls instance or generator
    """
    flatten_gen = itertools.chain.from_iterable(sequence_list)
    return cls(flatten_gen) if cls else flatten_gen


def find_peak_item(sequence, include_extremes=False):
    """
    Find peak item in sequence, yield each peak item with index.
    Example:
        >>> list(find_peak_item([13, 4, 5, 3, 6, 9, 4]))
        [(1, 4), (2, 5), (3, 3), (5, 9)]

    :param sequence: sequence
    :param include_extremes: include extreme points
    """
    iters = enumerate(sequence)
    index, left_p = next(iters)
    if include_extremes:
        yield index, left_p

    index, cur_p = next(iters)
    right_p = cur_p
    for r_idx, right_p in iters:
        if cmp(left_p, cur_p) * cmp(cur_p, right_p) < 0:
            yield index, cur_p
        left_p, cur_p, index = cur_p, right_p, r_idx

    if include_extremes:
        yield index, right_p


def filter_n(func_or_none, sequence, n=1):
    """
    Return n items in sequence which match func_or_none.

    :param func_or_none: judge function or None to choice items that are True
    :param sequence: iterable sequence
    :param n: choice n items at most
    """
    func = func_or_none or (lambda x: x)
    i_seq = iter(i for i in sequence if func(i))
    for i in range(n):
        yield next(i_seq)


def every_n(sequence, n=1):
    """
    Iterate every n items in sequence.

    :param sequence: iterable sequence
    :param n: n items to iterate
    """
    i_sequence = iter(sequence)
    return zip(*[i_sequence for i in range(n)])


def iter_chunk(sequence, size):
    """
    Iterate sequence as chunks

    :param sequence: iterable sequence
    :param size: size of every chunk
    """
    length = len(sequence)
    for i in range(0, length, size):
        yield sequence[i: i + size]


def safe_max(sequence, default=None):
    """
    Return the max value from sequence, if sequence is empty,
    return the default value.

    :param sequence: iterable sequence
    :param default: default value
    """
    i_seq = iter(sequence)
    try:
        value = next(i_seq)
    except StopIteration:
        return default

    return max(itertools.chain(sequence, [value]))


def safe_min(sequence, default=None):
    """
    Return the min value from sequence, if sequence is empty,
    return the default value.

    :param sequence: iterable sequence
    :param default: default value
    """
    i_seq = iter(sequence)
    try:
        value = next(i_seq)
    except StopIteration:
        return default

    return min(itertools.chain(sequence, [value]))



#!/usr/bin/env python
# encoding: utf-8

"""
Debug helpers
"""

import functools
import pdb


def pdb_break(func):
    """
    set a debug break when function call
    :param func: function
    """
    @functools.wraps(func)
    def _(*args, **kwg):
        pdb.set_trace()
        return func(*args, **kwg)
    return _


def debug_call_trace(loger=None, name=None):
    """
    Trace a fun call with loger.
    :param loger: logging.Logger
    :param name: function name
    """
    def _(func):
        fname = name or func.func_name

        @functools.wraps(func)
        def f(*args, **kwg):
            fid = id(args) ^ id(kwg)
            loger.debug("%s[%d] call, args: %s, kwg: %s", fname, fid, args, kwg)
            try:
                res = func(*args, **kwg)
                loger.debug("%s[%s] return: %s", fname, fid, res)
                return res
            except Exception as err:
                loger.debug("%s[%d] got error: %s", fname, fid, err)
                raise
        return f
    return _


def display_profile_stats(filename=None, sort_fields=(), p_amount=()):
    import cProfile
    from pstats import Stats

    def profiler(func):
        @functools.wraps(func)
        def wrapper(*args, **kwg):
            f = func
            res = None
            try:
                cProfile.runctx("res = f(*args, **kwg)", globals(), locals(), filename)
                return res
            finally:
                if filename:
                    pstats = Stats(filename)
                    pstats.sort_stats(*sort_fields)
                    pstats.print_stats(*p_amount)
        return wrapper
    return profiler



#!/usr/bin/env python
# encoding: utf-8

from ycyc.base.decoratorutils import *
from ycyc.base.adapter import main_entry
from ycyc.base.allowfail import AllowFail
from ycyc.base.lazyutils import lazy_init
from ycyc.base.typeutils import freezed_attrs
from ycyc.base.filetools import oserror_format



#!/usr/bin/env python
# encoding: utf-8
"""
This package provided some utils and tools for developer.
The difference between utils and tools is that tool can
use other utils module is this package but util did not.
It only allow use standard libraries whatever utils or
tools.Remember named a util module ends with utils.
"""



#!/usr/bin/env python
# encoding: utf-8

from contextlib import contextmanager
import six
import logging
import threading
import time

logger = logging.getLogger(__name__)


@contextmanager
def catch(errors=Exception, reraise=None, callback=logger.warning):
    """
    A context manager to catch the exceptions raised from block.
    :param errors: exception or exceptions tuple
    :param reraise: reraise a new exception from catched exception(default: None)
    :param callback: callback when catched a exception(default: logger.warning)
    """
    exec_info = {
        "callback_returned": None,
        "exception": None,
        "ok": True,
    }
    try:
        yield exec_info
    except errors as err:
        exec_info["exception"] = err
        exec_info["ok"] = False
        if callback:
            exec_info["callback_returned"] = callback(err)
        if reraise:
            six.raise_from(reraise, err)


@contextmanager
def subprocessor(*args, **kwg):
    """
    With new subprocess call,terminate it when exit context
    :param args: argument pass to subprocess.Popen
    :param kwg: key word argument pass to subprocess.Popen
    """
    from subprocess import Popen
    processor = Popen(*args, **kwg)
    try:
        yield processor
    finally:
        if processor.poll() is None:
            processor.terminate()
            processor.wait()


@contextmanager
def companion(
    target, auto_start=True, auto_join=True, process=threading.Thread,
):
    """
    Run the target function as block companion.
    :param target: callable object
    :param auto_start: start thread immediately
    :param auto_join: join thread when exit
    """
    processor = process(target=target)
    processor.daemon = True
    if auto_start:
        processor.start()
    try:
        yield processor
    finally:
        if processor.is_alive():
            processor.join()


@contextmanager
def timeout(seconds, interval=None, ticks=None):
    """
    Send KeyboardInterrupt to main thread to terminate it and
    convert it as RuntimeError if timeout.
    :param seconds: timeout seconds
    :param interval: poll interval
    :param ticks: CPU-Bound thread check interval
    """
    import os
    import signal
    import sys

    signal_finished = False
    start = time.time()
    interval = interval or 0.1
    old_ticks = sys.getcheckinterval()

    cur_thread = threading.current_thread()
    assert cur_thread.name == "MainThread"

    def poll_signal():
        now = time.time()
        while not signal_finished and now - start < seconds:
            time.sleep(interval)
            now = time.time()
        if not signal_finished:
            os.kill(os.getpid(), signal.SIGINT)

    with companion(poll_signal, auto_start=False) as poll_thread:
        if seconds > 0:
            poll_thread.start()

        if ticks is not None:
            sys.setcheckinterval(ticks)
        try:
            yield
        except KeyboardInterrupt:
            now = time.time()
            if now - start >= seconds > 0:
                raise RuntimeError("timeout")
            raise
        finally:
            signal_finished = True
            if ticks is not None:
                sys.setcheckinterval(old_ticks)


@contextmanager
def nothing(*args, **kwg):
    """
    Just a place holder.
    """
    yield


@contextmanager
def atlast(func, force=False):
    """
    Run func at last
    """
    try:
        yield
        func()
    except Exception:
        if force:
            func()
        raise


@contextmanager
def heartbeat(target, interval, auto_start=True, auto_join=True):
    """
    Run target as heartbeat.
    :param target: callable object
    :param interval: sleep interval
    :param auto_start: start thread immediately
    :param auto_join: join thread when exit
    """
    enable = True

    def check():
        while enable:
            target()
            time.sleep(interval)

    with companion(
        check, auto_start=auto_start, auto_join=auto_join,
    ) as processor:
        try:
            yield processor
        finally:
            enable = False



#!/usr/bin/env python
# encoding: utf-8
"""
AllowFail is a tool that you can use it to catch the exceptions for
a block as a context manager or a function as a decorator.
"""

import logging
from collections import namedtuple
from functools import wraps

__all__ = ("AllowFailResult", "AllowFail")
AllowFailResult = namedtuple("AllowFailResult", ["result", "exception"])


class AllowFail(object):
    """
    As a context manager:
    >>> with AllowFail("not important block"):
    ...     not_important_operator()

    As a decorator:
    >>> @AllowFail("not important function")
    ... def not_important_function():
    ...     pass
    It will packing the function returned result as AllowFailResult.
    AllowFailResult is a subclass of tuple which the first item is
    the real result of function call(set None when crashed) and the
    second item is the catched exception (set None when success).
    Call example:
    >>> result, exception = not_important_function()
    ... if exception is not None:
    ...     do_rockball_operator()

    >>> result = not_important_function()
    >>> if result.exception is not None:
    ...     do_rockball_operator()
    """
    logger = logging.root

    def __init__(self, label, on_error=None, logger=None, exc_info=False):
        """
        AllowFail constructor
        :param label: label pass to error_handler
        :param on_error: error_handler(default: AllowFail.on_error)
        :param logger: logger used in AllowFail.on_error
        :param exc_info: indicate AllowFail.on_error if log the execute info
        """
        self.label = label
        self.error_handler = on_error or self.on_error
        self.logger = logger or self.logger
        self.exc_info = exc_info

    def on_error(self, label, err):
        """
        Default on error handler
        :param label: label for logger
        :param err: exception
        """
        self.logger.warning("%s: %s", label, err, exc_info=self.exc_info)

    def __enter__(self):
        """
        Use instance as a context manager to protect a block
        :return: self
        """
        return self

    def __exit__(self, typ, val, trbk):
        """
        Catch exceptions
        :param typ: excrption type
        :param val: excrption
        :param trbk: traceback
        :return: True for catch
        """
        if typ:
            if val is None:
                val = typ()
            self.error_handler(self.label, val)

        # catch exceptions
        return True

    def __call__(self, func):
        """
        Use instance as a decorator to protect a function call
        :param func: function
        :return: protect function wrapper
        """
        @wraps(func)
        def protect(*args, **kwg):
            try:
                exception = None
                result = func(*args, **kwg)
            except Exception as err:
                exception = err
                result = None

            if exception:
                f_name = str(func)
                if hasattr(func, "__name__"):
                    f_name = func.__name__
                elif hasattr(func, "func_name"):
                    f_name = func.func_name
                else:
                    f_name = str(func)

                with AllowFail(
                        "On error handler %s" % f_name,
                        logger=self.logger, exc_info=self.exc_info):
                    self.error_handler(self.label, err)

            return AllowFailResult(result=result, exception=exception)
        return protect



#!/usr/bin/env python
# encoding: utf-8

"""
A adapter tool module
"""

import collections
import sys
import functools
import types

from ycyc.base import funcutils


class ObjAsDictAdapter(collections.Mapping):
    """
    Provided a adapter that allow visit a object as dict
    """

    def __init__(self, obj):
        self.__object = obj

    def __getitem__(self, name):
        try:
            return getattr(self.__object, name)
        except AttributeError:
            raise KeyError(name)

    def __setitem__(self, name, value):
        setattr(self.__object, name, value)

    def __iter__(self):
        return iter(dir(self.__object))

    def __len__(self):
        return len(dir(self.__object))


class DictAsObjAdapter(object):
    """
    Provided a adapter that allow visit a dict as object
    """

    def __init__(self, dct):
        self.__dict = dct

    def __getattr__(self, name):
        return self.__dict[name]

    def __setattr__(self, name, value):
        self.__dict[name] = value


class MappingMixin:

    def attrs_dict(self):
        return ObjAsDictAdapter(self)


def main_entry(main):
    """
    Call function `main` immediately if is in __main__ and exit
    """
    @functools.wraps(main)
    def wraped_main(*args, **kwg):
        result = main(*args, **kwg)
        return int(0 if result is None else result)

    if main.__module__ == "__main__":
        sys.exit(
            wraped_main()
            if main.func_code.co_argcount == 0
            else wraped_main(sys.argv)
        )

    return wraped_main


def proxy(obj):
    """
    A proxy to protect obj attributes will not be reassign
    """
    return type(
        "ProxyObject", (object,),
        {"__getattribute__": lambda _, k: getattr(obj, k)}
    )()


class DynamicClosure(object):

    def __init__(self, envs=()):
        from ycyc.base.iterutils import dict_merge

        self.environment = dict_merge(envs)

    def __call__(self, func, args, kwg):
        return func(self.environment.copy(), *args, **kwg)


def dynamic_closure(func, *args, **kwg):
    frame = funcutils.parent_frame()
    closure = DynamicClosure([frame.f_locals, frame.f_globals])
    return closure(func, args, kwg)


def attr_link(name):
    def getter(self):
        return getattr(self, name)

    def setter(self, value):
        return setattr(self, name, value)

    def deleter(self):
        return delattr(self, name)

    return property(getter, setter, deleter)



#!/usr/bin/env python
# encoding: utf-8

from ycyc.base.typeutils import SimpleExceptions

SimpleExceptions = SimpleExceptions()
OperateError = SimpleExceptions.OperateError
RangeError = SimpleExceptions.RangeError
EmptyRange = None


class ARange(object):
    def __init__(self, start, end, include_start=True, include_end=True):
        if end < start:
            raise RangeError

        self.start = start
        self.end = end
        self.include_start = bool(include_start)
        self.include_end = bool(include_end)

    def __str__(self):
        return "{lb}{start}..{end}{rb}".format(
            lb="[" if self.include_start else "(",
            start=self.start, end=self.end,
            rb="]" if self.include_end else ")",
        )

    def __repr__(self):
        return (
            "{clsname}({start}, {end}, "
            "include_start={include_start}, include_end={include_end})"
        ).format(
            clsname=self.__class__.__name__,
            start=repr(self.start),
            end=repr(self.end),
            include_start=repr(self.include_start),
            include_end=repr(self.include_end),
        )

    def __contains__(self, point):
        if point is None:
            return False

        if self.start < point < self.end:
            return True
        if self.include_start and point == self.start:
            return True
        if self.include_end and point == self.end:
            return True
        return False

    def __and__(self, other):
        if self.start > other.end:
            return EmptyRange
        if self.end < other.start:
            return EmptyRange

        if other.start in self:
            start = other.start
            include_start = other.include_start
        elif self.start in other:
            start = self.start
            include_start = self.include_start
        else:
            raise OperateError

        if other.end in self:
            end = other.end
            include_end = other.include_end
        elif self.end in other:
            end = self.end
            include_end = self.include_end
        else:
            raise OperateError

        return ARange(start, end, include_start, include_end)

    def __add__(self, other):
        return ARange(
            self.start + other.start,
            self.end + other.end,
            include_start=self.include_start and other.include_start,
            include_end=self.include_end and other.include_end,
        )

    def __sub__(self, other):
        return ARange(
            self.start - other.start,
            self.end - other.end,
            include_start=self.include_start and other.include_start,
            include_end=self.include_end and other.include_end,
        )

    def __mul__(self, other):
        vals = [
            self.start * other.start,
            self.start * other.end,
            self.end * other.start,
            self.end * other.end,
        ]
        return ARange(
            min(vals),
            max(vals),
            include_start=self.include_start and other.include_start,
            include_end=self.include_end and other.include_end,
        )

    def is_after_of(self, point):
        if point < self.start:
            return True
        if not self.include_start and self.start == point:
            return True
        return False

    def is_before_of(self, point):
        if point > self.end:
            return True
        if not self.include_end and self.end == point:
            return True
        return False

    def has_sub_range(self, other):
        if self.is_after_of(other.start):
            if other.start != self.start or other.include_start:
                return False
        if self.is_before_of(other.end):
            if other.end != self.end or other.include_end:
                return False
        return True

    def is_follow_by(self, other):
        if self.end != other.start:
            return False
        if self.include_end == other.include_start:
            return False
        return True

    def sampling(self, step):
        val = self.start if self.include_start else self.start + step
        include_end = self.include_end
        end = self.end

        while val < end:
            yield val
            val += step

        if val == end and include_end:
            yield val



#!/usr/bin/env python
# encoding: utf-8

from collections import deque


class FilterList(deque):
    def __str__(self):
        return str(list(self))

    def __repr__(self):
        return "%s(%r)" % (self.__class__.__name__, list(self))

    def filter(self, func):
        """
        Return a new FilterList which filter by func
        """
        return self.__class__(filter(func, self))

    def exclude(self, func):
        """
        Return a new FilterList which exclude by func
        """
        return self.filter(lambda x: not func(x))

    def first(self, default=None):
        """
        Return the first item if existed, otherwise return default
        """
        return self[0] if self else default

    def last(self, default=None):
        """
        Return the last item if existed, otherwise return default
        """
        return self[-1] if self else default



import heapq as heapq_op
from ycyc.base.iterutils import getattrs


class HeapItem(object):

    def __init__(self, value, heap):
        self.value = value
        self.heap = heap

    def __str__(self):
        return str(self.value)

    def __repr__(self):
        return "%s(%r, %r)" % (
            self.__class__.__name__, self.value, self.heap,
        )

    def __cmp__(self, other):
        return self.heap.compare_items(self, other)


class Heap(object):

    def __init__(self, iterable=(), cmp_attrs=(), reverse=False):
        self.data = list(
            HeapItem(i, self)
            for i in iterable
        )
        self.cmp_attrs = cmp_attrs
        self.reverse = reverse
        heapq_op.heapify(self.data)

    def compare_items(self, item1, item2):
        value = cmp(
            getattrs(item1.value, self.cmp_attrs),
            getattrs(item2.value, self.cmp_attrs),
        )
        return value * -1 if self.reverse else value

    def __getitem__(self, index):
        item = self.data[index]
        return item.value

    def __iter__(self):
        return iter(self.headn(len(self.data)))

    def push(self, value):
        item = HeapItem(value, self)
        heapq_op.heappush(self.data, item)

    def pop(self):
        item = heapq_op.heappop(self.data)
        return item.value

    def edge_out(self, value):
        item = heapq_op.heapreplace(self.data, HeapItem(value, self))
        return item.value

    def headn(self, n):
        return [
            i.value for i in heapq_op.nsmallest(n, self.data)
        ]

    def tailn(self, n):
        values = [
            i.value for i in heapq_op.nlargest(n, self.data)
        ]
        return values[::-1]



#!/usr/bin/env python
# encoding: utf-8

import os


def simple_table(row, col, cell_factory):
    """
    Create and return a simple table, like: [[1, 2, 3], [4, 5, 6], [7, 8, 9]]
    """
    return [
        [cell_factory(i, j) for j in range(col)]
        for i in range(row)
    ]


class BaseTable(object):
    def __init__(self, row, col):
        self.height = row
        self.width = col

    @property
    def rows(self):
        return (
            [self[i, j] for i in range(self.width)]
            for j in range(self.height)
        )

    @property
    def cols(self):
        return (
            [self[i, j] for i in range(self.height)]
            for j in range(self.width)
        )


class Table(BaseTable):
    def __init__(self, row, col, init_val=None):
        self.table = [
            [init_val for j in range(col)]
            for i in range(row)
        ]
        super(Table, self).__init__(row, col)

    def __getitem__(self, idx):
        row, col = idx
        return self.table[row][col]

    def __setitem__(self, idx, val):
        row, col = idx
        self.table[row][col] = val


class SparseTable(BaseTable):
    def __init__(self, row, col, init_val=None):
        self.init_val = init_val
        self.table = {}
        super(SparseTable, self).__init__(row, col)

    def __getitem__(self, idx):
        r, c = idx
        row = self.table.get(r)
        if row is None:
            return self.init_val
        return row.get(c, self.init_val)

    def __setitem__(self, idx, val):
        r, c = idx
        row = self.table.get(r)
        if row is None:
            row = self.table[r] = {}
        row[c] = val



#!/usr/bin/env python
# encoding: utf-8

import sys
from HTMLParser import HTMLParser
import re
from collections import namedtuple
import argparse

from ycyc.base.iterutils import dict_merge
from ycyc.base.filetools import safe_open_for_read, safe_open_for_write
from ycyc.base.adapter import main_entry


class InlineStyleConvertor(HTMLParser):
    StyleAttribute = namedtuple("StyleAttribute", ["key", "value"])
    Style = namedtuple("Style", ["selector", "attributes"])
    StyleLevelOrders = map(re.compile, [
        r"#\w+$",
        r"\.\w+$",
        r"\b\w+$",
        r"\*$",
    ])

    def __init__(self, output, style_txt, ignore=False):
        self.output = output
        self.styles = self.format_style(style_txt)
        self.is_ignore = ignore
        self.is_in_body = None
        HTMLParser.__init__(self)

    @classmethod
    def level_of(cls, selector):
        for i, rex in enumerate(cls.StyleLevelOrders):
            if rex.match(selector):
                break
        return i

    @classmethod
    def cmp_style(cls, a, b):
        return cmp(cls.level_of(a.selector), cls.level_of(b.selector))

    def sort_styles(self, styles):
        styles.sort(self.cmp_style, reverse=True)
        return styles

    def format_style(self, style_txt):
        style_txt = re.sub(r"/\*(\s|.)*?\*/", "", style_txt, re.M)
        styles = [
            i.split("{")
            for i in style_txt.split("}")
            if i.strip()
        ]
        results = {}
        for selector, attrs in styles:
            selector = selector.strip()
            if not selector:
                continue
            attributes = []
            for attr in attrs.split(";"):
                attr = attr.strip()
                if not attr:
                    continue
                k, v = attr.split(":")
                attributes.append(self.StyleAttribute(k.strip(), v.strip()))
            results[selector] = self.Style(selector, attributes)
        return results

    def css_dict(self, selector):
        css = {}
        styles = self.styles.get(selector)
        if styles:
            for attr in styles.attributes:
                css[attr.key] = attr.value
        return css

    def handle_starttag(self, tag, attrs):
        if tag == "body":
            self.is_in_body = True

        attrs = dict(attrs)

        if self.is_in_body:
            inline_style = attrs.pop("style", "")
            if self.is_ignore:
                styles_list = []
            else:
                styles_list = [dict(
                    i.strip().split(":")
                    for i in inline_style.split(";")
                    if i.strip()
                )]

            eid = attrs.get("id")
            if eid:
                styles_list.append(self.css_dict("#%s" % eid))

            for cls in attrs.get("class", "").split():
                styles_list.append(self.css_dict(".%s" % cls))

            styles_list.append(self.css_dict(tag))
            styles_list.append(self.css_dict("*"))

            if styles_list:
                styles = dict_merge(styles_list)
            if styles:
                styles_items = styles.items()
                styles_items.sort(lambda a, b: cmp(a[0], b[0]))
                attrs["style"] = "; ".join("%s:%s" % i for i in styles_items)

        attr_str = " ".join("%s=\"%s\"" % i for i in attrs.items())
        self.output.write(
            "<{tag}{attrs}>".format(
                tag=tag, attrs=" %s" % attr_str if attr_str else "",
            )
        )

    def handle_endtag(self, tag):
        if tag == "body":
            self.is_in_body = False

        self.output.write(
            "</{tag}>".format(tag=tag)
        )

    def handle_data(self, data):
        self.output.write(data)

    def feed(self, html):
        self.is_in_body = False
        return HTMLParser.feed(self, html)


@main_entry
def main():
    arg_parser = argparse.ArgumentParser()
    arg_parser.add_argument("base_file")
    arg_parser.add_argument(
        "-s", "--style",
        help="css file"
    )
    arg_parser.add_argument(
        "-o", "--output", default=None,
        help="output file",
    )
    arg_parser.add_argument(
        "-i", "--ignore", action="store_true", default=False,
        help="ignore inline style",
    )
    args = arg_parser.parse_args()

    with safe_open_for_read(args.base_file) as fp:
        html = fp.read()

    if args.style:
        with safe_open_for_read(args.style) as fp:
            style = fp.read()
    else:
        style = ""

    if args.output:
        fp = safe_open_for_write(args.output)
    else:
        fp = sys.stdout
    with fp:
        parser = InlineStyleConvertor(fp, style, args.ignore)
        parser.feed(html)



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase
import inspect

import mock

from tests import mock_patches
from ycyc.base import adapter


class TestObjAsDictAdapter(TestCase):

    class Parent(object):
        pclsval = 1

        def __init__(self):
            self.pinsval = 2

    class Child(Parent):
        cclsval = 3

        def __init__(self):
            super(TestObjAsDictAdapter.Child, self).__init__()
            self.cinsval = 4

        def __getattr__(self, name):
            if name == "dynamicval":
                return 5

    def test_base_method(self):
        parent = self.Parent()
        parent_dict = adapter.ObjAsDictAdapter(parent)

        self.assertEqual(parent_dict["pclsval"], parent.pclsval)
        self.assertEqual(parent_dict["pinsval"], parent.pinsval)
        self.assertIn("pclsval", iter(parent_dict))
        self.assertIn("pinsval", iter(parent_dict))

    def test_inherit(self):
        child = self.Child()
        child_dict = adapter.ObjAsDictAdapter(child)

        self.assertEqual(child_dict["pclsval"], child.pclsval)
        self.assertEqual(child_dict["pinsval"], child.pinsval)
        self.assertEqual(child_dict["cclsval"], child.cclsval)
        self.assertEqual(child_dict["cinsval"], child.cinsval)
        self.assertIn("pclsval", iter(child_dict))
        self.assertIn("pinsval", iter(child_dict))
        self.assertIn("cclsval", iter(child_dict))
        self.assertIn("cinsval", iter(child_dict))

    def test_dynamic_attr(self):
        child = self.Child()
        child_dict = adapter.ObjAsDictAdapter(child)

        self.assertEqual(child_dict["dynamicval"], child.dynamicval)
        self.assertNotIn("dynamicval", iter(child_dict))

        child.dynamicval = 6
        self.assertEqual(child_dict["dynamicval"], child.dynamicval)
        self.assertIn("dynamicval", iter(child_dict))

    def test_dict_interface(self):
        child = self.Child()
        parent = self.Parent()
        child_dict = adapter.ObjAsDictAdapter(child)
        parent_dict = adapter.ObjAsDictAdapter(parent)

        self.assertEqual(parent_dict.get("cinsval"), None)
        self.assertEqual(
            parent_dict.get("cinsval", child_dict["cinsval"]),
            child_dict.get("cinsval"))

        self.assertGreater(len(parent_dict), 0)
        self.assertEqual(parent_dict.keys(), list(parent_dict.iterkeys()))
        self.assertEqual(parent_dict.values(), list(parent_dict.itervalues()))
        self.assertEqual(parent_dict.items(), list(parent_dict.iteritems()))
        self.assertEqual(len(parent_dict), len(list(parent_dict)))


class TestMainEntry(TestCase):

    @property
    def main_mock(self):
        main_mock = mock.MagicMock()
        main_mock.func_name = main_mock.__name__ = "main_mock"
        main_mock.func_doc = main_mock.__doc__ = "main_mock doc"
        main_mock.func_globals = mock.MagicMock()
        main_mock.func_defaults = mock.MagicMock()
        main_mock.func_code = mock.MagicMock()
        return main_mock

    def test_usage(self):
        with mock_patches(
            "ycyc.base.adapter.sys",
        ) as patches:
            main_mock = self.main_mock
            main_mock.return_value = 1

            new_main = adapter.main_entry(main_mock)
            self.assertEqual(new_main(1, 2, 3), main_mock.return_value)
            main_mock.assert_called_once_with(1, 2, 3)

        with mock_patches(
            "ycyc.base.adapter.sys",
        ) as patches:
            main_mock = self.main_mock
            main_mock.__module__ = "__main__"
            main_mock.func_code.co_argcount = 0
            main_mock.return_value = 1

            new_main = adapter.main_entry(main_mock)

            main_mock.assert_called_once_with()
            patches.sys.exit.assert_called_once_with(main_mock.return_value)

        with mock_patches(
            "ycyc.base.adapter.sys",
        ) as patches:
            patches.sys.argv = [id(patches)]
            main_mock = self.main_mock
            main_mock.__module__ = "__main__"
            main_mock.func_code.co_argcount = 1
            main_mock.return_value = None

            new_main = adapter.main_entry(main_mock)

            main_mock.assert_called_once_with(patches.sys.argv)
            patches.sys.exit.assert_called_once_with(0)


class TestProxy(TestCase):

    def test_usage(self):
        mock_object = mock.MagicMock()
        mock_object.value = "lyc"
        proxy_object = adapter.proxy(mock_object)

        self.assertEqual(mock_object.value, proxy_object.value)

        value = proxy_object.value = 123
        self.assertNotEqual(mock_object.value, value)
        self.assertEqual(mock_object.value, proxy_object.value)

        value = mock_object.value = 123
        self.assertEqual(mock_object.value, value)
        self.assertEqual(mock_object.value, proxy_object.value)

        @adapter.proxy
        class Consts(object):
            A = 1
            B = 2

        Consts.A = 2
        self.assertEqual(Consts.A, 1)

        Consts.B = 3
        self.assertEqual(Consts.B, 2)


class TestDynamicClosure(TestCase):

    def test_usage(self):
        def check(env):
            self.assertEqual(env["l_value"], g_value)

        def invoker(value):
            l_value = value
            adapter.dynamic_closure(check)

        g_value = 1
        invoker(g_value)
        g_value = 2
        invoker(g_value)


class TestAttrLink(TestCase):

    def test_usage(self):
        class A(object):
            a = 1
            b = adapter.attr_link("a")

        a = A()
        self.assertIs(a.a, a.b)
        a.a = 2
        self.assertIs(a.b, 2)
        a.b = 3
        self.assertIs(a.a, 3)



# encoding: utf-8

from itertools import chain


class RedisKit(object):
    @classmethod
    def factory(cls, *base_args, **base_kwargs):
        def constructor(*args, **kwargs):
            kwargs.update(base_kwargs)
            return cls(*chain(base_args, args), **kwargs)
        return constructor



#!/usr/bin/env python
# encoding: utf-8

import ast
import logging

from ycyc.collections.tagmaps import TagMaps
from ycyc.libs.txtgen import TxtGenerator
from ycyc.base.iterutils import getfirst, mkparts
from ycyc.base.adapter import main_entry

logger = logging.getLogger(__name__)


class Analysis(object):
    NodeAnalysisMaps = TagMaps()

    def __init__(self):
        self.analysis_result = TxtGenerator()

    def node_name(self, node):
        return node.__class__.__name__

    def analysis(self, path):
        with open(path, "rt") as fp:
            codes = fp.read()

        return self.analysis_ast(ast.parse(codes))

    def analysis_ast(self, module_ast):
        for node in module_ast.body:
            method = self.NodeAnalysisMaps.get(self.node_name(node))
            if method:
                method(self, node)
                self.analysis_result.writeline("")

        result = self.analysis_result.getval()
        self.analysis_result.clear()
        return result

    @NodeAnalysisMaps.register(NodeAnalysisMaps.DefaultKey)
    def visit_default(self, node):
        return False

    def var_name(self, node):
        if isinstance(node, ast.Name):
            return node.id
        elif isinstance(node, ast.Attribute):
            return "%s.%s" % (self.var_name(node.value), node.attr)
        raise TypeError("%s" % type(node))

    def get_docstring(self, node):
        if not isinstance(node, (ast.FunctionDef, ast.ClassDef)):
            return ""
        ds_node = getfirst(node.body)
        if not isinstance(ds_node, ast.Expr):
            return ""
        ds_node = ds_node.value
        if not isinstance(ds_node, ast.Str):
            return ""
        return ds_node.s.lstrip("\n").rstrip()

    @NodeAnalysisMaps.register("Assign")
    def visit_assign(self, node):
        flag = False
        for target in node.targets:
            if isinstance(target, ast.Attribute):
                continue
            flag = True
            self.analysis_result.writeline(
                "var: {name}",
                lineno=target.lineno,
                name=self.var_name(target),
            )
        return flag

    @NodeAnalysisMaps.register("FunctionDef")
    def visit_functiondef(self, node):
        node_args = node.args
        args_lst = [a.id for a in node_args.args]
        if node_args.vararg:
            args_lst.append("*%s" % node_args.vararg)
        if node_args.kwarg:
            args_lst.append("**%s" % node_args.kwarg)

        decorators = [
            "%s(*)" % d.attr if isinstance(d, ast.Call) else self.var_name(d)
            for d in node.decorator_list
        ]
        decorators.reverse()

        self.analysis_result.writeline(
            "function: {name}({args}){decorators}",
            lineno=node.lineno,
            name=node.name,
            args=", ".join(args_lst),
            decorators=(" @%s" % decorators) if decorators else "",
        )
        docstring = self.get_docstring(node)
        if docstring:
            self.analysis_result.writeline(
                "{docstring}\n--- docstring ---",
                docstring=docstring,
            )

    @NodeAnalysisMaps.register("ClassDef")
    def visit_classdef(self, node):
        basestr = ""
        if node.bases:
            basestr = "(%s)" % ", ".join([
                self.var_name(b) for b in node.bases
            ])

        self.analysis_result.writeline(
            "class: {name}{basestr}",
            name=node.name,
            basestr=basestr,
        )
        docstring = self.get_docstring(node)
        if docstring:
            self.analysis_result.writeline(
                "{docstring}\n--- docstring ---",
                docstring=docstring,
            )

        for node in node.body:
            method = self.NodeAnalysisMaps.get(self.node_name(node))
            if method:
                self.analysis_result.writeline("")
                self.analysis_result.write("+ ")
                method(self, node)


@main_entry
def main(argv):
    _, paths = mkparts(argv, [1])

    analysisor = Analysis()
    for p in paths:
        print(p)
        print("===== result =====")
        print(analysisor.analysis(p))



#!/usr/bin/env python
# coding=utf-8

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

try:
    from ycyc import __version__
except ImportError:
    __version__ = "unknown version"


from setuptools import find_packages

dirname = os.path.dirname(__file__)

standard_exclude = ["*.py", "*.pyc", "*$py.class", "*~", ".*", "*.bak"]


setup(
    name='ycyc',
    version=__version__,
    description='a library by LYC',
    long_description=open('pypi.rst', 'a+').read(),
    author='Liu Yicong',
    author_email='imyikong@gmail.com',
    packages=find_packages(exclude=["tests.*", "tests"]),
    install_requires=['six==1.9.0'],
    license='BSD',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
)



#!/usr/bin/env python
# encoding: utf-8

import subprocess

from ycyc.base import decoratorutils
from ycyc.base import contextutils


class Command(object):
    @classmethod
    def subprocess_args(cls, name, cmd_args, popen_kwargs):
        cmds = [name]
        cmds.extend(map(str, cmd_args))
        popen_kwargs.setdefault("shell", False)
        return cmds, popen_kwargs

    def __init__(self, name):
        self.name = name

    def __call__(self, *args, **kwargs):
        cmds, kwargs = self.subprocess_args(self.name, args, kwargs)
        return subprocess.Popen(cmds, **kwargs)

    def check_call(self, *args, **kwargs):
        cmds, kwargs = self.subprocess_args(self.name, args, kwargs)
        return subprocess.check_call(cmds, **kwargs)

    def check_output(self, *args, **kwargs):
        cmds, kwargs = self.subprocess_args(self.name, args, kwargs)
        return subprocess.check_output(cmds, **kwargs)

    def subprocessor(self, *args, **kwargs):
        cmds, kwargs = self.subprocess_args(self.name, args, kwargs)
        kwargs.setdefault("stdout", subprocess.PIPE)
        kwargs.setdefault("stderr", subprocess.PIPE)

        return contextutils.subprocessor(cmds, **kwargs)


@decoratorutils.call_immediately()
class ShellCommands(object):
    """
    >>> ShellCommands.echo("hello lyc")
    hello lyc
    """
    def __getattr__(self, name):
        return Command(name)



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase

from ycyc.base import funcutils


class TestObjectHelper(TestCase):
    def test_is_magic_method(self):
        class Foo(object):
            def __str__(self):
                return "foo"

        foo = Foo()
        self.assertTrue(funcutils.is_magic_method(Foo.__str__))
        self.assertTrue(funcutils.is_magic_method(foo.__str__))

    def test_set_default_attr(self):
        class Foo(object):
            def __str__(self):
                return "foo"

        foo = Foo()
        self.assertFalse(hasattr(foo, "foobarbaz"))
        self.assertTrue(funcutils.set_default_attr(
            foo, "foobarbaz", True
        ))
        self.assertTrue(foo.foobarbaz)
        self.assertTrue(funcutils.set_default_attr(
            foo, "foobarbaz", False
        ))
        self.assertTrue(foo.foobarbaz)

    def test_iter_attrs(self):
        class Foo(object):
            Public = 0
            _Protected = 1
            __Privated = 2

            def __init__(self):
                self.public = 4
                self._protected = 5
                self.__privated = 6

            def get_privated(self):
                return self.__privated

            @property
            def protected(self):
                return self._protected

        foo = Foo()

        self.assertDictEqual(
            dict(funcutils.iter_attrs(foo)),
            {
                "Public": 0, "public": 4, "protected": 5,
            }
        )
        self.assertDictEqual(
            dict(funcutils.iter_attrs(foo, public_only=False)),
            {
                "Public": 0, "_Protected": 1, "public": 4,
                "protected": 5, "_protected": 5,
            }
        )
        self.assertDictEqual(
            dict(funcutils.iter_attrs(foo, include_fields={"_Protected"})),
            {
                "Public": 0, "_Protected": 1, "public": 4,
                "protected": 5,
            }
        )
        self.assertDictEqual(
            dict(funcutils.iter_attrs(foo, exclude_fields={"protected"})),
            {
                "Public": 0, "public": 4,
            }
        )
        result = dict(funcutils.iter_attrs(foo, exclude_methods=False))
        self.assertEqual(result["get_privated"], foo.get_privated)

        self.assertIn(
            "_Protected",
            dict(funcutils.iter_attrs(
                foo,
                include_fields={"_Protected"},
                exclude_fields={"_Protected"}
            ))
        )


class TestParentFrame(TestCase):
    def test_usage(self):
        import sys
        import thread

        frames = sys._current_frames()
        current_frame = frames[thread.get_ident()]
        frame = current_frame.f_back

        self.assertIs(frame, funcutils.parent_frame())


class TestExportModule(TestCase):
    def test_funcutils(self):
        locals_env = locals()
        export_module = funcutils.export_module(funcutils.__name__)
        for attr in dir(funcutils):
            if not attr.startswith("_"):
                self.assertIs(getattr(funcutils, attr), locals_env[attr])
            else:
                self.assertNotIn(attr, locals_env)

    def test_os(self):
        import os
        locals_env = locals()
        funcutils.export_module("os")
        for attr in os.__all__:
            self.assertIs(getattr(os, attr), locals_env[attr])



#!/usr/bin/env python
# encoding: utf-8


class Marker(object):
    __slots__ = ["name", "value"]

    def __init__(self, name=NotImplemented, value=NotImplemented):
        self.name = (
            "Mark[%s]" % id(self) if name is NotImplemented else name
        )
        self.value = value

    def __setattr__(self, name, val):
        if hasattr(self, name):
            raise AttributeError("attribute %s is not writable" % name)
        return super(Marker, self).__setattr__(name, val)

    def __nonzero__(self):
        return bool(self.value)

    def __eq__(self, other):
        if not isinstance(other, Marker):
            return False
        return self.name == other.name

    def __str__(self):
        return self.name

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, repr(self.name))

Marker.Undefined = Marker("undefined", None)
Marker.Missed = Marker("missed", None)
Marker.Disabled = Marker("disabled", None)

try:
    from ycyc.base.lazyutils import lazy_init

    class SimpleMarkers(object):
        @lazy_init
        def __getattr__(self, name):
            return Marker(name)
except ImportError:
    pass



#!/usr/bin/env python
# encoding: utf-8

from ycyc.base import lazyutils

from unittest import TestCase


class TestLazyEnv(TestCase):
    def test_attr_usage(self):
        env = lazyutils.LazyEnv()
        env.a = lambda: global_a

        with self.assertRaisesRegexp(NameError, r"\bglobal_a\b"):
            env.a

        global_a = []
        self.assertIs(env.a, global_a)
        global_a = {}
        self.assertIsNot(env.a, global_a)

        env.a = global_a
        self.assertIs(env.a, global_a)

        global_a = ()
        env.a = lambda: global_a
        self.assertIs(env.a, global_a)

    def test_item_usage(self):
        import types

        env = lazyutils.LazyEnv()
        env["a"] = lambda: global_a

        self.assertIsInstance(env.a, types.LambdaType)
        self.assertIsInstance(env["a"], types.LambdaType)

        env.a = lambda: global_a
        with self.assertRaisesRegexp(NameError, r"\bglobal_a\b"):
            env.a

        global_a = []
        self.assertIs(env.a, global_a)


class TestLazyImport(TestCase):
    def test_usage(self):
        lazy_os = lazyutils.lazy_import("os")
        import os
        for attr in dir(os):
            self.assertIs(
                getattr(os, attr),
                getattr(lazy_os, attr),
            )

        lazy_lazyutils = lazyutils.lazy_import(lazyutils.__name__)
        for attr in dir(lazyutils):
            self.assertIs(
                getattr(lazyutils, attr),
                getattr(lazy_lazyutils, attr),
            )

        lazy_akulamatata = lazyutils.lazy_import("noting.akulamatata")
        with self.assertRaisesRegexp(
            ImportError, "No module named noting.akulamatata"
        ):
            lazy_akulamatata.fail()


def TestLazyKit(TestCase):
    def test_usage(self):
        gen = (i for i in range(3))
        lazy_list = lazyutils.LazyKit(lambda: list(gen))
        self.assertEqual(next(gen), 0)
        self.assertEqual(lazy_list[1], 1)
        self.assertEqual(lazy_list[2], 2)
        self.assertSetEqual(set(dir([])) - set(dir(lazy_list)), set())



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase

import mock

from ycyc.base import decoratorutils


class TestCachedProperty(TestCase):
    def test_cachedproperty(self):
        from datetime import datetime

        obj_mock = mock.MagicMock(returnval=datetime.now())

        class Object(object):
            @decoratorutils.cachedproperty
            def first_visited_time(self):
                return obj_mock()

        obj = Object()
        self.assertIs(obj.first_visited_time, obj.first_visited_time)
        self.assertEqual(obj_mock.call_count, 1)

        obj = Object()
        self.assertIs(obj.first_visited_time, obj.first_visited_time)
        self.assertEqual(obj_mock.call_count, 2)


class TestChainingMethod(TestCase):
    def test_chainingmethod(self):
        class MockCls(object):
            def __init__(self):
                self.called = []

            @decoratorutils.chainingmethod
            def position_arg(self, val):
                self.called.append(("position_arg", val))

            @decoratorutils.chainingmethod
            def keyword_arg(self, val):
                self.called.append(("keyword_arg", {"val": val}))

            @decoratorutils.chainingmethod
            def free_args(self, *args, **kwg):
                self.called.append(("free_args", args, kwg))


        mockobj = MockCls()
        mockobj.position_arg(1).keyword_arg(val=2).free_args(3, val=4)

        self.assertEqual(mockobj.called.pop(), ("free_args", (3,), {"val": 4}))
        self.assertEqual(mockobj.called.pop(), ("keyword_arg", {"val": 2}))
        self.assertEqual(mockobj.called.pop(), ("position_arg", 1))


class TestRetry(TestCase):
    def mockfunc(self, iterator):
        try:
            n = next(iterator)
            raise ValueError("%s" % n)
        except StopIteration:
            return None

    def test_retry_useage(self):
        @decoratorutils.retry(3)
        def mockfunc3(iterator):
            return self.mockfunc(iterator)

        @decoratorutils.retry(0)
        def mockfunc0(iterator):
            return self.mockfunc(iterator)

        with self.assertRaisesRegexp(ValueError, "can not less than 0"):
            @decoratorutils.retry(-1)
            def mockfunc(iterator):
                return self.mockfunc(iterator)

        self.assertIsNone(mockfunc3(iter(range(1))))
        self.assertIsNone(mockfunc3(iter(range(2))))
        self.assertIsNone(mockfunc3(iter(range(3))))
        with self.assertRaisesRegexp(Exception, "3"):
            mockfunc3(iter(range(4)))
        with self.assertRaisesRegexp(Exception, "3"):
            mockfunc3(iter(range(5)))

        self.assertIsNone(mockfunc0(iter(range(1))))
        self.assertIsNone(mockfunc0(iter(range(2))))
        self.assertIsNone(mockfunc0(iter(range(3))))
        self.assertIsNone(mockfunc0(iter(range(4))))
        self.assertIsNone(mockfunc0(iter(range(5))))


class TestWithManager(TestCase):
    def test_withmanager(self):
        ctxmgr_cls = mock.MagicMock()
        ctxmgr_obj = mock.MagicMock()
        ctxmgr_cls.return_value = ctxmgr_obj
        enter_mock = mock.MagicMock()
        exit_mock = mock.MagicMock()
        setattr(ctxmgr_obj, "__enter__", enter_mock)
        setattr(ctxmgr_obj, "__exit__", exit_mock)

        @decoratorutils.withmanager(ctxmgr_cls, 1, 2, k=3)
        def func1(k):
            return k + 1

        @decoratorutils.withmanager(lambda: ctxmgr_obj)
        def func2(k):
            return k + 1
        self.assertEqual(func1(4), 5)
        self.assertEqual(ctxmgr_cls.call_count, 1)
        self.assertEqual(ctxmgr_cls.call_args[0], (1, 2))
        self.assertEqual(ctxmgr_cls.call_args[1], {"k": 3})
        self.assertEqual(ctxmgr_obj.call_count, 0)
        self.assertEqual(enter_mock.call_count, 1)
        self.assertEqual(exit_mock.call_count, 1)

        self.assertEqual(func2(5), 6)
        self.assertEqual(ctxmgr_cls.call_count, 1)
        self.assertEqual(ctxmgr_obj.call_count, 0)
        self.assertEqual(enter_mock.call_count, 2)
        self.assertEqual(exit_mock.call_count, 2)


class TestOnErrorReturn(TestCase):
    def test_usage(self):
        callback_mock = mock.MagicMock()

        @decoratorutils.onerror_return(None, StopIteration, callback_mock)
        def gen_next(gen):
            return next(gen)

        gen = (i for i in range(2))
        self.assertEqual(gen_next(gen), 0)
        self.assertEqual(gen_next(gen), 1)
        self.assertEqual(callback_mock.call_count, 0)
        self.assertEqual(gen_next(gen), None)
        self.assertEqual(callback_mock.call_count, 1)

        gen = (i / 0 for i in range(2))
        with self.assertRaises(ZeroDivisionError):
            gen_next(gen)

        self.assertEqual(callback_mock.call_count, 1)


class TestCallImmediately(TestCase):
    def test_usage(self):
        num = 10
        init_val=0

        @decoratorutils.call_immediately()
        def val_lst():
            return [init_val for i in range(num)]

        self.assertListEqual(val_lst, [
            init_val for i in range(num)
        ])


class TestMethodDecorator(TestCase):
    def test_usage(self):
        def add_one(func):
            return lambda: 1 + func()

        class AClass(object):
            def __init__(self, val):
                self.val = val

            @decoratorutils.method_decorator(add_one)
            def get_value(self):
                return self.val

        a = AClass(1)
        b = AClass(5)

        self.assertEqual(a.get_value(), 2)
        self.assertEqual(b.get_value(), 6)


class TestWithAttr(TestCase):
    def test_usage(self):
        @decoratorutils.withattr(alters_data=True, debug=True)
        def test_func(val):
            return val + 1

        self.assertTrue(test_func.alters_data)
        self.assertTrue(test_func.debug)
        self.assertEqual(test_func(0), 1)


class TestAllowUnboundMethod(TestCase):
    def test_usage(self):
        invokers = []

        class TestCls(object):
            @decoratorutils.allow_unbound_method
            def save_invoker(invoker):
                invokers.append(invoker)

        TestCls.save_invoker()
        self.assertEqual(TestCls, invokers.pop())

        test_obj = TestCls()
        test_obj.save_invoker()
        self.assertEqual(test_obj, invokers.pop())


class TestClassProperty(TestCase):
    def test_usage(self):
        class Model(object):
            @decoratorutils.classproperty
            def val(cls):
                return 1

        self.assertEqual(Model.val, 1)
        model = Model()
        self.assertEqual(model.val, 1)




#!/usr/bin/env python
# encoding: utf-8

try:
    import readline
    import rlcompleter
except ImportError:
    print("auto complete failed to install")
else:
    readline.parse_and_bind("tab: complete")
    print("auto complete installed")



#!/usr/bin/env python
# encoding: utf-8

import time

from unittest import TestCase
import mock

from ycyc.base.contextutils import (
    catch, timeout, atlast, companion, heartbeat
)


class TestCatch(TestCase):
    def test_catch_errors(self):
        with catch():
            raise Exception()

        with catch(ZeroDivisionError):
            1 / 0

        with catch((KeyError, IndexError)):
            list()[1]
            dict()["nothing"]

    def test_raise(self):
        with self.assertRaises(KeyError):
            with catch(IndexError):
                dict()["nothing"]

        with self.assertRaises(IndexError):
            with catch(KeyError, reraise=IndexError):
                dict()["nothing"]

        with self.assertRaises(ValueError):
            with catch((ZeroDivisionError, IndexError), reraise=ValueError):
                1 / 0

        with self.assertRaises(ValueError):
            with catch((ZeroDivisionError, IndexError), reraise=ValueError):
                list()[1]

        with self.assertRaises(ValueError):
            try:
                valerr = ValueError()
                with catch(ZeroDivisionError, reraise=valerr):
                    1 / 0

            except ValueError as err:
                self.assertIs(err, valerr)
                raise

    def test_callback(self):
        results = []

        with catch((ZeroDivisionError, IndexError), callback=results.append):
            1 / 0
        self.assertIsInstance(results.pop(), ZeroDivisionError)

        with catch((ZeroDivisionError, IndexError), callback=results.append):
            list()[1]
        self.assertIsInstance(results.pop(), IndexError)

    def test_exec_info(self):
        with catch() as exec_info:
            self.assertTrue(exec_info["ok"])
            self.assertIsNone(exec_info["exception"])
            self.assertIsNone(exec_info["callback_returned"])

        self.assertTrue(exec_info["ok"])
        self.assertIsNone(exec_info["exception"])
        self.assertIsNone(exec_info["callback_returned"])

        callback = mock.MagicMock(return_value=1)
        with catch(NotImplementedError, callback=callback) as exec_info:
            raise NotImplementedError

        self.assertFalse(exec_info["ok"])
        self.assertIsInstance(exec_info["exception"], NotImplementedError)
        self.assertEqual(exec_info["callback_returned"], callback.return_value)


class TestCompanion(TestCase):
    def test_usage(self):
        mock_target = mock.MagicMock()
        with companion(mock_target) as thread:
            pass
        mock_target.assert_called_once_with()

        mock_target = mock.MagicMock()
        with companion(mock_target, auto_start=False) as thread:
            self.assertEqual(mock_target.call_count, 0)
            thread.start()
            time.sleep(0.001)
            self.assertEqual(mock_target.call_count, 1)


class TestTimeout(TestCase):
    def test_usage(self):
        import sys

        ticks = sys.getcheckinterval()

        with self.assertRaisesRegexp(RuntimeError, "timeout"):
            start = time.time()
            with timeout(0.01, 0.01):
                time.sleep(1)
        self.assertLess(time.time() - start, 1)
        self.assertEqual(ticks, sys.getcheckinterval())

        with self.assertRaises(KeyboardInterrupt):
            with timeout(0.01, 0.01):
                raise KeyboardInterrupt
        self.assertEqual(ticks, sys.getcheckinterval())

        with self.assertRaisesRegexp(RuntimeError, "timeout"):
            start = time.time()
            with timeout(0.01, 0.01, 10):
                while True:
                    pass
        self.assertLess(time.time() - start, 0.03)
        self.assertEqual(ticks, sys.getcheckinterval())

        # 0 for forever
        with timeout(0, 0.001):
            time.sleep(0.01)
        self.assertEqual(ticks, sys.getcheckinterval())


class TestAtlast(TestCase):
    def test_usage(self):
        func = mock.MagicMock()

        with atlast(func):
            self.assertEqual(func.call_count, 0)
        self.assertEqual(func.call_count, 1)

        with self.assertRaises(ZeroDivisionError):
            with atlast(func):
                1 / 0
        self.assertEqual(func.call_count, 1)

        with self.assertRaises(ZeroDivisionError):
            with atlast(func, force=True):
                1 / 0
        self.assertEqual(func.call_count, 2)


class TestHeartbeat(TestCase):
    def test_usage(self):
        mock_touch = mock.MagicMock()

        self.assertEqual(mock_touch.call_count, 0)
        with heartbeat(mock_touch, 0.01):
            call_count = mock_touch.call_count
            time.sleep(0.05)
            self.assertLess(call_count, mock_touch.call_count)
            call_count = mock_touch.call_count
            time.sleep(0.05)
            self.assertLess(call_count, mock_touch.call_count)

        call_count = mock_touch.call_count
        time.sleep(0.02)
        self.assertEqual(call_count, mock_touch.call_count)




from collections import deque
from itertools import repeat
import os


class Matrix(object):

    @classmethod
    def from_table(cls, table, init_val=0):
        data = []
        row_n = 0
        col_n = 0

        for rn, r in enumerate(table):
            row = []
            for cn, c in enumerate(r):
                row.append(c)

                if col_n <= cn:
                    col_n = cn + 1
            data.append(row)
            if row_n <= rn:
                row_n = rn + 1

        return cls(row_n=row_n, col_n=col_n, matrix=data, init_val=init_val)

    @classmethod
    def check_size(
        cls, matrix, row_checker=lambda x: True, col_checker=lambda x: True
    ):
        if not row_checker(matrix.row_n) or not col_checker(matrix.col_n):
            raise ValueError("matrix size error: [%d, %d]" % (
                matrix.row_n, matrix.col_n,
            ))

    def __init__(self, row_n, col_n, matrix=None, init_val=0):
        self.init_val = init_val
        self.row_n = int(row_n)
        self.col_n = int(col_n)

        matrix = matrix or (
            repeat(init_val, col_n)
            for r in range(row_n)
        )
        self.data = deque((
            deque((c for c in r), maxlen=col_n)
            for r in matrix
        ), maxlen=row_n)

        self.make_sure_matrix_size()

    def make_sure_matrix_size(self):
        row_n = self.row_n
        col_n = self.col_n
        init_val = self.init_val
        data = self.data

        self.check_size(
            self,
            lambda x: x > 0,
            lambda x: x > 0,
        )

        for r in range(row_n):
            try:
                col = data[r]
                col_len = len(col)
                if col_len < col_n:
                    col.extend(repeat(init_val, col_n - col_len))
            except IndexError:
                data.append(deque(repeat(init_val, col_n)))

    def visit_with(self, callback):
        for i in range(self.row_n):
            for j in range(self.col_n):
                callback(i, j, self[i, j])

    def __iter__(self):
        return iter(self.data)

    def __getitem__(self, idx):
        if isinstance(idx, (tuple, list)):
            row_n, col_n = idx
            return self.data[row_n][col_n]
        return self.data[idx]

    def __setitem__(self, idx, val):
        if not isinstance(idx, (tuple, list)):
            raise ValueError("index type error: %s" % idx)

        row_n, col_n = idx
        if row_n >= self.row_n or col_n >= self.col_n:
            raise IndexError("index error: [%s, %s]" % (row_n, col_n))

        self.data[row_n][col_n] = val

    def __str__(self):
        output = []
        max_width = 0

        for r in self:
            line = []
            for c in r:
                element_str = str(c)
                line.append(element_str)

                element_str_len = len(element_str)
                if element_str_len > max_width:
                    max_width = element_str_len

            else:
                output.append(line)

        template = "%%%ds" % (max_width + 1)
        return os.linesep.join(
            ",".join(template % c for c in r)
            for r in output
        )

    def __eq__(self, other):
        if self.row_n != other.row_n or self.col_n != other.col_n:
            return False

        for mr, tr, r in zip(self, other, range(self.row_n)):
            for mc, tc, c in zip(mr, tr, range(self.col_n)):
                if mc != tc:
                    return False
            else:
                if c != self.col_n - 1:
                    return False
        else:
            if r != self.row_n - 1:
                return False
        return True

    def __add__(self, other):
        self.check_size(
            other,
            lambda x: self.row_n == x,
            lambda x: self.col_n == x,
        )

        matrix = Matrix(self.row_n, self.col_n)

        def add(i, j, val):
            matrix[i, j] = val + other[i, j]

        self.visit_with(add)
        return matrix

    def __sub__(self, other):
        self.check_size(
            other,
            lambda x: self.row_n == x,
            lambda x: self.col_n == x,
        )

        matrix = Matrix(self.row_n, self.col_n)

        def sub(i, j, val):
            matrix[i, j] = val - other[i, j]

        self.visit_with(sub)
        return matrix

    def __repr__(self):
        return str(self)

    def __mul__(self, other):
        self.check_size(
            other,
            lambda x: self.col_n == x,
        )

        matrix = Matrix(self.row_n, other.col_n)
        for i in range(self.row_n):
            for j in range(other.col_n):
                elememt = 0

                for k in range(self.col_n):
                    elememt += self[i, k] * other[k, j]
                else:
                    matrix[i, j] = elememt

        return matrix

    def __rmul__(self, other):
        matrix = Matrix(self.row_n, self.col_n, self)
        for i in range(self.row_n):
            for j in range(self.col_n):
                matrix[i, j] *= other
        return matrix

    def transpose_matrix(self):
        matrix = Matrix(self.col_n, self.row_n)
        for i in range(self.row_n):
            for j in range(self.col_n):
                matrix[j, i] = self[i, j]
        return matrix



#!/usr/bin/env python
# encoding: utf-8

import ast
import astunparse


class BrainHoleDiff(ast.NodeTransformer):
    def has_symbol(self, node):
        return node.id == "x"

    def expr_parse(self, expr):
        node = ast.parse(expr)
        return node.body[0].value

    def visit_Call(self, node):
        func_name = node.func.id
        if not self.has_symbol(node.args[0]):
            return self.generic_visit(node)

        if func_name == "sin":
            node.func.id = "cos"
        elif func_name == "cos":
            node = self.expr_parse("-sin(x)")
        elif func_name == "ln":
            node = self.expr_parse("1.0/x")
        elif func_name == "pow":
            times = node.args[1].n
            node = self.expr_parse("%s*pow(x, %s)" % (times, times - 1))

        return node

    def __call__(self, expr):
        ast_node = ast.parse(expr)
        ast_node = self.visit(ast_node)
        return astunparse.unparse(ast_node)

diff = BrainHoleDiff()

print diff("3*sin(x)-cos(x)") # 3*cos(x) + sin(x)
print diff("ln(x)") # 1.0/x
print diff("3*ln(x)+sin(x)-pow(x, 2)+cos(x)")
print eval(diff("pow(x, 3)-pow(x, 2)"), {"x": 1}) # 1
